package org.memoriesdreamsandreflections.waipt_procedures.testutils ;


import java.io.File ;
import java.io.IOException ;
import java.util.ArrayList ;
import java.util.List ;

import org.jboss.arquillian.container.test.api.Testable ;
import org.jboss.shrinkwrap.api.ArchivePaths ;
import org.jboss.shrinkwrap.api.ShrinkWrap ;
import org.jboss.shrinkwrap.api.exporter.ZipExporter ;
import org.jboss.shrinkwrap.api.spec.EnterpriseArchive ;
import org.jboss.shrinkwrap.api.spec.JavaArchive ;
import org.jboss.shrinkwrap.api.spec.WebArchive ;
import org.jboss.shrinkwrap.resolver.api.DependencyResolvers ;
import org.jboss.shrinkwrap.resolver.api.maven.EffectivePomMavenDependencyResolver ;
import org.jboss.shrinkwrap.resolver.api.maven.MavenDependency ;
import org.jboss.shrinkwrap.resolver.api.maven.MavenDependencyResolver ;
import org.jboss.shrinkwrap.resolver.api.maven.filter.CombinedFilter ;
import org.jboss.shrinkwrap.resolver.api.maven.filter.DependencyFilter ;
import org.jboss.shrinkwrap.resolver.api.maven.filter.ExclusionFilter ;
import org.jboss.shrinkwrap.resolver.api.maven.filter.ScopeFilter ;
import org.memoriesdreamsandreflections.waipt_common.Globals ;
import org.memoriesdreamsandreflections.waipt_common.testutils.IntegrationTestUtil ;
import org.memoriesdreamsandreflections.waipt_common.testutils.WaiptConfigUtil ;
import org.memoriesdreamsandreflections.waipt_common.utils.PathUtil ;
import org.testng.Assert ;

public class ArchiveUtilExtended
{

	/**
	 * 
	 * Builds the EAR once per test thread, and incorporates PID into ear name. Saves considerable time in integration test runs.
	 * @return
	 */
	public static EnterpriseArchive createTestArchive()
	{
		return createTestArchive( false ) ;
	}
	/**
	 * 
	 * Builds the EAR once per test thread, and incorporates PID into ear name. Saves considerable time in integration test runs.
	 * @return
	 */
	public static EnterpriseArchive createTestArchive( boolean forceNewDeployment )
	{
		EnterpriseArchive l_ea = null ;
		String l_tempDir = System.getProperty(  Globals.INTEGRATION_TEMP_DIR, "/tmp" ) ;
		String l_earTemplate = l_tempDir + "/test-procedures-" ; 
		File l_earFile = IntegrationTestUtil.produceIntegrationTestEarFile( l_earTemplate );
		
		if ( l_earFile.exists() && ! forceNewDeployment )
		{
			l_ea = ShrinkWrap.createFromZipFile( EnterpriseArchive.class, l_earFile ) ;
			/*
			 * Verify that at least one archive is testable
			 */
			WebArchive l_wa = l_ea.getAsType( WebArchive.class, ArchivePaths.create("test.war") );
			Assert.assertTrue( Testable.isArchiveToTest( l_wa ), "No testable archive" ) ;
			return l_ea ;
		}
		else
		{
			PathUtil l_path = new PathUtil( System.getProperty( "user.dir" ) ) ;
			
			EffectivePomMavenDependencyResolver resolver = DependencyResolvers.use( MavenDependencyResolver.class )
				.loadEffectivePom( l_path.getPomXml() )
				.importAllDependencies() ;
			
			l_ea = ShrinkWrap.create(  EnterpriseArchive.class, l_earFile.getName() ) ;
	
			l_ea.addAsLibraries( dependencies_( resolver ) ) ;
	
			l_ea.addAsLibrary( extensions_( resolver ) )  ;
	
			l_ea.addAsLibrary( persistenceXml_() )  ;
			
			l_ea.addAsLibrary( entities_( resolver ) ) ;
			
			l_ea.addAsLibrary( are_( resolver ) ) ;
	
			l_ea.addAsManifestResource( "waimap-ds.xml" ) ;
	
			WebArchive l_wa = Testable.archiveToTest( 
				ShrinkWrap.create( WebArchive.class, "test.war" )
					.addPackages( true, "org.memoriesdreamsandreflections.waipt_procedures" )
					.addAsWebInfResource( "beans.xml", "beans.xml" ) ) ;
			l_ea.addAsModule( l_wa ) ;
			
			l_ea.addAsModule( lockbox_() );
			
			l_ea.addAsManifestResource( "jboss-deployment-structure.xml" );
			
			
			try
			{
				if ( ! forceNewDeployment )
				{
					l_ea.as(  ZipExporter.class ).exportTo( l_earFile, true );
					System.out.println( "@@@@TEST ARCHIVE WRITTEN: " + l_earFile.getCanonicalPath() );
				}
			}
			catch(Exception e)
			{
				e.printStackTrace() ;
			}
		}
		
		try
		{
			WaiptConfigUtil.createWaiptConfig( l_tempDir ) ;
			WaiptConfigUtil.createLockboxConfig( l_tempDir ) ;
		}
		catch( IOException e)
		{
			Assert.fail("Exception", e);
		}
		
		return l_ea ;

	}
	
	static WebArchive lockbox_()
	{
		File l_lockboxWarFile = null ;
		try
		{
			l_lockboxWarFile = IntegrationTestUtil.produceArchiveFileFromMavenCoords( "org.memoriesdreamsandreflections:waipt-lockbox-war:war:0.1" );
			
		}
		catch(Exception e)
		{
			Assert.fail( "Exception " + e, e );
		}
		WebArchive l_lockboxWar = ShrinkWrap.createFromZipFile( WebArchive.class, l_lockboxWarFile ) ;
		l_lockboxWar.delete( "WEB-INF/lib");
		return l_lockboxWar ;
	}

	static File[] dependencies_( EffectivePomMavenDependencyResolver resolver )
	{
		/*
		 * 
		 * Shrinkwrap seems to run into some trouble when integration tests are run from the 'waipt-jee' level; 
		 * In that context, 'waipt-ejb-entities' and 'waipt-jar-are' were each being written to the EAR lib 
		 * twice: once with a normal version and once with a timestamp. This causes Integration testing to fail. 
		 * It's most likely caused by the way I'm using "resolver.resolveAsFiles()" but I'm not certain.
		 * Anyway I got things working by excluding the problematic dependencies from the resolver and then
		 * explicitly adding those same dependencies subsequently using 'addAsLibrary'. 
		 */
		CombinedFilter l_cf = new CombinedFilter(
			new ScopeFilter( "compile", "runtime", "test" ),
			new ExclusionFilter( "org.memoriesdreamsandreflections:waipt-ejb-entities:0.1" ),
			new ExclusionFilter( "org.memoriesdreamsandreflections:waipt-jar-are:0.1" ),
			new ExclusionFilter( "org.memoriesdreamsandreflections:waipt-lockbox-war:war:0.1" ) //,
					//new ArqShrinkExclusionFilter() 
		) ;
		return resolver.resolveAsFiles( l_cf ) ;
	}
	
	static JavaArchive persistenceXml_()
	{
		JavaArchive l_persistenceXml = ShrinkWrap.create( JavaArchive.class, "waipt-ee-persistence.jar" );
		l_persistenceXml.addAsManifestResource("META-INF/test-persistence.xml", "persistence.xml") ;
		return l_persistenceXml ;
	}
	
	static JavaArchive extensions_( EffectivePomMavenDependencyResolver resolver )
	{
		List<JavaArchive> l_jaList = new ArrayList<JavaArchive>( resolver.resolveAs( JavaArchive.class, new DependencyFilter( "org.memoriesdreamsandreflections:waipt-jee-cdi-extensions:jar:0.1" ) ) ) ;
		return l_jaList.get( 0 ) ;
	}
	
	static JavaArchive entities_( EffectivePomMavenDependencyResolver resolver )
	{
		List<JavaArchive> l_jaList = new ArrayList<JavaArchive>( resolver.resolveAs( JavaArchive.class, new DependencyFilter( "org.memoriesdreamsandreflections:waipt-ejb-entities:0.1" ) ) ) ;
		return l_jaList.get( 0 ) ;
	}
	
	static JavaArchive are_( EffectivePomMavenDependencyResolver resolver )
	{
		List<JavaArchive> l_jaList = new ArrayList<JavaArchive>( resolver.resolveAs( JavaArchive.class, new DependencyFilter( "org.memoriesdreamsandreflections:waipt-jar-are:0.1" ) ) ) ;
		return l_jaList.get( 0 ) ;
	}
	
	static class ArqShrinkExclusionFilter
	extends ExclusionFilter
	{
		public ArqShrinkExclusionFilter()
		{
			super( "" ) ;
		}

		@Override
		public boolean accept( MavenDependency element )
		{
			return element.coordinates().indexOf( "org.jboss.shrinkwrap" ) == -1
			&& element.coordinates().indexOf( "org.jboss.arquillian" ) == -1
			&& element.coordinates().indexOf( "testng" ) == -1 ;
		}
	}
	
}
